//Ifndef-define-endif is used to prevent multiple inclusion of the same header file
#ifndef CALCULATOR_H
#define CALCULATOR_H

#include <SFML/System.hpp>
#include <SFML/Graphics.hpp>
#include <vector>
#include <sstream>

//What operation are we performing?
enum Operator {
    ADD=0xF0,
    SUBTRACT=0xF1,
    MULTIPLY=0xF2,
    DIVIDE=0xF3,
    NONE=0xFF
};

//Sign of the inserted number
enum Sign {
    POSITIVE=+1,
    NEGATIVE=-1
};

//Which button did the user click?
enum ButtonCode {
    ZERO=0,
    ONE=1,
    TWO=2,
    THREE=3,
    FOUR=4,
    FIVE=5,
    SIX=6,
    SEVEN=7,
    EIGHT=8,
    NINE=9,
    SWAP_SIGN,
    INSERT_DOT,
    PLUS=0xF0,
    MINUS=0xF1,
    PER=0xF2,
    SOLIDUS=0xF3,
    EQUAL,
    CLEAR
};

//Define the calculator class
class Calculator {
    //Private means only class methods and few other things can access these elements
    private:
        sf::RenderWindow Window; //Window on which we'll render everything
        std::vector<std::pair<sf::Shape, sf::String>> RegularButtons; //Buttons we'll have to click (numbers, sign swap and dot)
        std::vector<std::pair<sf::Shape, sf::String>> OpButtons; //+, -, *, /
        std::vector<std::pair<sf::Shape, sf::String>> ExButtons; //=,
        std::pair<sf::Shape, sf::String> Display; //Calculator display
        sf::String SignString; //+ - sign

        long double PrecValue, Value; //Value inserted
        bool DotInserted; //Has . already been inserted in this value?
        Operator Op; //Which operation do we have to perform?
        size_t InsertedCharacters; //Characters (ciphers or dot) inserted
        //Internal variables
        std::ostringstream stream;
        Sign sign;
        bool IsFirstAlreadyReplaced;
        bool ResultOutputted;

        //Function used to elaborate button press
        void InsertCharacter(ButtonCode bc); //Press a button

    //Public members can be freely accessed
    public:
        //Class Constructor (Function that is automatically called when you istantiate this class)
        Calculator();
        //Class Destructor (Function that is automatically called when the object goes "out of scope")
        ~Calculator();
        //Handle events, display output
        void Start();
};

#endif // CALCULATOR_H
